@@ -1,22 +1,22 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015, Vispy Development Team.
+# Copyright (c) Vispy Development Team. All Rights Reserved.
 # Distributed under the (new) BSD License. See LICENSE.txt for more info.
 
 """
 Base code for the Qt backends. Note that this is *not* (anymore) a
 backend by itself! One has to explicitly use either PySide, PyQt4 or
-PyQt5. Note that the automatic backend selection prefers a GUI toolkit
-that is already imported.
+PySide2, PyQt5. Note that the automatic backend selection prefers
+a GUI toolkit that is already imported.
 
-The _pyside, _pyqt4 and _pyqt5 modules will import * from this module,
-and also keep a ref to the module object. Note that if two of the
-backends are used, this module is actually reloaded. This is a sorts
-of poor mans "subclassing" to get a working version for both backends
-using the same code.
+The _pyside, _pyqt4, _pyside2 and _pyqt5 modules will import * from
+this module, and also keep a ref to the module object. Note that if
+two of the backends are used, this module is actually reloaded. This
+is a sorts of poor mans "subclassing" to get a working version for
+both backends using the same code.
 
-Note that it is strongly discouraged to use the PySide/PyQt4/PyQt5
-backends simultaneously. It is known to cause unpredictable behavior
-and segfaults.
+Note that it is strongly discouraged to use the
+PySide/PyQt4/PySide2/PyQt5 backends simultaneously. It is known to
+cause unpredictable behavior and segfaults.
 """
 
 from __future__ import division
@@ -26,12 +26,14 @@ import os
 import sys
 import atexit
 import ctypes
+from distutils.version import LooseVersion
 
 from ...util import logger
 from ..base import (BaseApplicationBackend, BaseCanvasBackend,
                     BaseTimerBackend)
 from ...util import keys
 from ...ext.six import text_type
+from ...ext.six import string_types
 from ... import config
 from . import qt_lib
 
@@ -54,7 +56,7 @@ elif sys.platform.startswith('win'):
 
 def _check_imports(lib):
     # Make sure no conflicting libraries have been imported.
-    libs = ['PyQt4', 'PyQt5', 'PySide']
+    libs = ['PyQt4', 'PyQt5', 'PySide', 'PySide2']
     libs.remove(lib)
     for lib2 in libs:
         lib2 += '.QtCore'
@@ -65,23 +67,42 @@ def _check_imports(lib):
 # Get what qt lib to try. This tells us wheter this module is imported
 # via _pyside or _pyqt4 or _pyqt5
 QGLWidget = object
+QT5_NEW_API = False
 if qt_lib == 'pyqt4':
     _check_imports('PyQt4')
     if not USE_EGL:
         from PyQt4.QtOpenGL import QGLWidget, QGLFormat
-    from PyQt4 import QtGui, QtCore
+    from PyQt4 import QtGui, QtCore, QtTest
     QWidget, QApplication = QtGui.QWidget, QtGui.QApplication  # Compat
 elif qt_lib == 'pyqt5':
     _check_imports('PyQt5')
     if not USE_EGL:
-        from PyQt5.QtOpenGL import QGLWidget, QGLFormat
-    from PyQt5 import QtGui, QtCore, QtWidgets
+        from PyQt5.QtCore import QT_VERSION_STR
+        if LooseVersion(QT_VERSION_STR) >= '5.4.0':
+            from PyQt5.QtWidgets import QOpenGLWidget as QGLWidget
+            from PyQt5.QtGui import QSurfaceFormat as QGLFormat
+            QT5_NEW_API = True
+        else:
+            from PyQt5.QtOpenGL import QGLWidget, QGLFormat
+    from PyQt5 import QtGui, QtCore, QtWidgets, QtTest
+    QWidget, QApplication = QtWidgets.QWidget, QtWidgets.QApplication  # Compat
+elif qt_lib == 'pyside2':
+    _check_imports('PySide2')
+    if not USE_EGL:
+        from PySide2.QtCore import __version__ as QT_VERSION_STR
+        if LooseVersion(QT_VERSION_STR) >= '5.4.0':
+            from PySide2.QtWidgets import QOpenGLWidget as QGLWidget
+            from PySide2.QtGui import QSurfaceFormat as QGLFormat
+            QT5_NEW_API = True
+        else:
+            from PySide2.QtOpenGL import QGLWidget, QGLFormat
+    from PySide2 import QtGui, QtCore, QtWidgets, QtTest
     QWidget, QApplication = QtWidgets.QWidget, QtWidgets.QApplication  # Compat
 elif qt_lib == 'pyside':
     _check_imports('PySide')
     if not USE_EGL:
         from PySide.QtOpenGL import QGLWidget, QGLFormat
-    from PySide import QtGui, QtCore
+    from PySide import QtGui, QtCore, QtTest
     QWidget, QApplication = QtGui.QWidget, QtGui.QApplication  # Compat
 elif qt_lib:
     raise RuntimeError("Invalid value for qt_lib %r." % qt_lib)
@@ -137,14 +158,45 @@ BUTTONMAP = {0: 0, 1: 1, 2: 2, 4: 3, 8: 4, 16: 5}
 
 
 # Properly log Qt messages
-# Also, ignore spam about tablet input
-def message_handler(msg_type, msg):
-    if msg == ("QCocoaView handleTabletEvent: This tablet device is "
-               "unknown (received no proximity event for it). Discarding "
-               "event."):
-        return
+def message_handler(*args):
+
+    if qt_lib in ("pyqt4", "pyside"):
+        msg_type, msg = args
+    elif qt_lib in ("pyqt5", "pyside2"):  # Is this correct for pyside2?
+        msg_type, context, msg = args
+    elif qt_lib:
+        raise RuntimeError("Invalid value for qt_lib %r." % qt_lib)
     else:
-        logger.warning(msg)
+        raise RuntimeError("Module backends._qt ",
+                           "should not be imported directly.")
+
+    BLACKLIST = [
+        # Ignore spam about tablet input
+        'QCocoaView handleTabletEvent: This tablet device is unknown',
+        # Not too sure why this warning is emitted when using
+        #   Spyder + PyQt5 + Vispy
+        #   https://github.com/vispy/vispy/issues/1787
+        # In either case, it is really annoying. We should filter it away
+        'QSocketNotifier: Multiple socket notifiers for same',
+    ]
+    for item in BLACKLIST:
+        if msg.startswith(item):
+            return
+
+    msg = msg.decode() if not isinstance(msg, string_types) else msg
+    logger.warning(msg)
+
+
+def use_shared_contexts():
+    """Enable context sharing for PyQt5 5.4+ API applications.
+
+    This is disabled by default for PyQt5 5.4+ due to occasional segmentation
+    faults and other issues when contexts are shared.
+
+    """
+    forced_env_var = os.getenv('VISPY_PYQT5_SHARE_CONTEXT', 'false').lower() == 'true'
+    return not QT5_NEW_API or forced_env_var
+
 
 try:
     QtCore.qInstallMsgHandler(message_handler)
@@ -163,7 +215,7 @@ capability = dict(  # things that can be set by the backend
     resizable=True,
     decorate=True,
     fullscreen=True,
-    context=True,
+    context=use_shared_contexts(),
     multi_window=True,
     scroll=True,
     parent=True,
@@ -179,15 +231,21 @@ def _set_config(c):
     glformat.setGreenBufferSize(c['green_size'])
     glformat.setBlueBufferSize(c['blue_size'])
     glformat.setAlphaBufferSize(c['alpha_size'])
-    glformat.setAccum(False)
-    glformat.setRgba(True)
-    glformat.setDoubleBuffer(True if c['double_buffer'] else False)
-    glformat.setDepth(True if c['depth_size'] else False)
+    if QT5_NEW_API:
+        # Qt5 >= 5.4.0 - below options automatically enabled if nonzero.
+        glformat.setSwapBehavior(glformat.DoubleBuffer if c['double_buffer']
+                                 else glformat.SingleBuffer)
+    else:
+        # Qt4 and Qt5 < 5.4.0 - buffers must be explicitly requested.
+        glformat.setAccum(False)
+        glformat.setRgba(True)
+        glformat.setDoubleBuffer(True if c['double_buffer'] else False)
+        glformat.setDepth(True if c['depth_size'] else False)
+        glformat.setStencil(True if c['stencil_size'] else False)
+        glformat.setSampleBuffers(True if c['samples'] else False)
     glformat.setDepthBufferSize(c['depth_size'] if c['depth_size'] else 0)
-    glformat.setStencil(True if c['stencil_size'] else False)
     glformat.setStencilBufferSize(c['stencil_size'] if c['stencil_size']
                                   else 0)
-    glformat.setSampleBuffers(True if c['samples'] else False)
     glformat.setSamples(c['samples'] if c['samples'] else 0)
     glformat.setStereo(c['stereo'])
     return glformat
@@ -199,6 +257,10 @@ class ApplicationBackend(BaseApplicationBackend):
 
     def __init__(self):
         BaseApplicationBackend.__init__(self)
+        # sharing is currently buggy and causes segmentation faults for tests with PyQt 5.6
+        if QT5_NEW_API and use_shared_contexts():
+            # For Qt5 >= 5.4.0 - Enable sharing of context between windows.
+            QApplication.setAttribute(QtCore.Qt.AA_ShareOpenGLContexts, True)
 
     def _vispy_get_backend_name(self):
         name = QtCore.__name__.split('.')[0]
@@ -230,9 +292,16 @@ class ApplicationBackend(BaseApplicationBackend):
         # Return
         return app
 
+    def _vispy_sleep(self, duration_sec):
+        QtTest.QTest.qWait(duration_sec * 1000)  # in ms
+
 
 # ------------------------------------------------------------------ canvas ---
 
+def _get_qpoint_pos(pos):
+    """Return the coordinates of a QPointF object."""
+    return pos.x(), pos.y()
+
 
 class QtBaseCanvasBackend(BaseCanvasBackend):
     """Base functionality of Qt backend. No OpenGL Stuff."""
@@ -258,6 +327,16 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
             self._fullscreen = True
         else:
             self._fullscreen = False
+
+        # must set physical size before setting visible or fullscreen
+        # operations may make the size invalid
+        if hasattr(self, 'devicePixelRatio'):
+            # handle high DPI displays in PyQt5
+            ratio = self.devicePixelRatio()
+        else:
+            ratio = 1
+        self._physical_size = (p.size[0] * ratio, p.size[1] * ratio)
+
         if not p.resizable:
             self.setFixedSize(self.size())
         if p.position is not None:
@@ -269,6 +348,32 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
         # avoid double events
         self._double_click_supported = True
 
+        try:
+            # see screen_changed docstring for more details
+            self.window().windowHandle().screenChanged.connect(self.screen_changed)
+        except AttributeError:
+            # either not PyQt5 backend or no parent window available
+            pass
+
+        # Activate touch and gesture.
+        # NOTE: we only activate touch on OS X because there seems to be
+        # problems on Ubuntu computers with touchscreen.
+        # See https://github.com/vispy/vispy/pull/1143
+        if sys.platform == 'darwin':
+            self.setAttribute(QtCore.Qt.WA_AcceptTouchEvents)
+            self.grabGesture(QtCore.Qt.PinchGesture)
+
+    def screen_changed(self, new_screen):
+        """Window moved from one display to another, resize canvas.
+
+        If display resolutions are the same this is essentially a no-op except for the redraw.
+        If the display resolutions differ (HiDPI versus regular displays) the canvas needs to
+        be redrawn to reset the physical size based on the current `devicePixelRatio()` and
+        redrawn with that new size.
+
+        """
+        self.resizeGL(*self._vispy_get_size())
+
     def _vispy_warmup(self):
         etime = time() + 0.25
         while time() < etime:
@@ -286,6 +391,14 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
         # Set size of the widget or window
         self.resize(w, h)
 
+    def _vispy_set_physical_size(self, w, h):
+        self._physical_size = (w, h)
+
+    def _vispy_get_physical_size(self):
+        if self._vispy_canvas is None:
+            return
+        return self._physical_size
+
     def _vispy_set_position(self, x, y):
         # Set location of the widget or window. May have no effect for widgets
         self.move(x, y)
@@ -391,6 +504,40 @@ class QtBaseCanvasBackend(BaseCanvasBackend):
     def keyReleaseEvent(self, ev):
         self._keyEvent(self._vispy_canvas.events.key_release, ev)
 
+    def event(self, ev):
+        out = super(QtBaseCanvasBackend, self).event(ev)
+        t = ev.type()
+        # Two-finger pinch.
+        if (t == QtCore.QEvent.TouchBegin):
+            self._vispy_canvas.events.touch(type='begin')
+        if (t == QtCore.QEvent.TouchEnd):
+            self._vispy_canvas.events.touch(type='end')
+        if (t == QtCore.QEvent.Gesture):
+            gesture = ev.gesture(QtCore.Qt.PinchGesture)
+            if gesture:
+                (x, y) = _get_qpoint_pos(gesture.centerPoint())
+                scale = gesture.scaleFactor()
+                last_scale = gesture.lastScaleFactor()
+                rotation = gesture.rotationAngle()
+                self._vispy_canvas.events.touch(type='pinch',
+                                                pos=(x, y),
+                                                last_pos=None,
+                                                scale=scale,
+                                                last_scale=last_scale,
+                                                rotation=rotation,
+                                                )
+        # General touch event.
+        elif (t == QtCore.QEvent.TouchUpdate):
+            points = ev.touchPoints()
+            # These variables are lists of (x, y) coordinates.
+            pos = [_get_qpoint_pos(p.pos()) for p in points]
+            lpos = [_get_qpoint_pos(p.lastPos()) for p in points]
+            self._vispy_canvas.events.touch(type='touch',
+                                            pos=pos,
+                                            last_pos=lpos,
+                                            )
+        return out
+
     def _keyEvent(self, func, ev):
         # evaluates the keycode of qt, and transform to vispy key.
         key = int(ev.key())
@@ -535,7 +682,7 @@ class CanvasBackendEgl(QtBaseCanvasBackend, QWidget):
             # screenshot and draw that for now ...
             # Further, QImage keeps a ref to the data that we pass, so
             # we need to use a static buffer to prevent memory leakage
-            from vispy import gloo
+            from ... import gloo
             import numpy as np
             if not hasattr(self, '_gl_buffer'):
                 self._gl_buffer = np.ones((3000 * 3000 * 4), np.uint8) * 255
@@ -581,25 +728,54 @@ class CanvasBackendDesktop(QtBaseCanvasBackend, QGLWidget):
                 raise RuntimeError('Cannot use vispy to share context and '
                                    'use built-in shareWidget.')
 
-        # first arg can be glformat, or a gl context
         if p.always_on_top or not p.decorate:
             hint = 0
             hint |= 0 if p.decorate else QtCore.Qt.FramelessWindowHint
             hint |= QtCore.Qt.WindowStaysOnTopHint if p.always_on_top else 0
         else:
             hint = QtCore.Qt.Widget  # can also be a window type
-        QGLWidget.__init__(self, glformat, p.parent, widget, hint)
+        if QT5_NEW_API:
+            # Qt5 >= 5.4.0 - sharing is automatic
+            QGLWidget.__init__(self, p.parent, hint)
+
+            # Need to create an offscreen surface so we can get GL parameters
+            # without opening/showing the Widget. PyQt5 >= 5.4 will create the
+            # valid context later when the widget is shown.
+            self._secondary_context = QtGui.QOpenGLContext()
+            self._secondary_context.setShareContext(self.context())
+            self._secondary_context.setFormat(glformat)
+            self._secondary_context.create()
+
+            self._surface = QtGui.QOffscreenSurface()
+            self._surface.setFormat(glformat)
+            self._surface.create()
+            self._secondary_context.makeCurrent(self._surface)
+        else:
+            # Qt4 and Qt5 < 5.4.0 - sharing is explicitly requested
+            QGLWidget.__init__(self, p.parent, widget, hint)
+            # unused with this API
+            self._secondary_context = None
+            self._surface = None
+
+        self.setFormat(glformat)
         self._initialized = True
-        if not self.isValid():
+        if not QT5_NEW_API and not self.isValid():
+            # On Qt5 >= 5.4.0, isValid is only true once the widget is shown
             raise RuntimeError('context could not be created')
-        self.setAutoBufferSwap(False)  # to make consistent with other backends
+        if not QT5_NEW_API:
+            # to make consistent with other backends
+            self.setAutoBufferSwap(False)
         self.setFocusPolicy(QtCore.Qt.WheelFocus)
 
     def _vispy_close(self):
         # Force the window or widget to shut down
         self.close()
         self.doneCurrent()
-        self.context().reset()
+        if not QT5_NEW_API:
+            self.context().reset()
+        if self._vispy_canvas is not None:
+            self._vispy_canvas.app.process_events()
+            self._vispy_canvas.app.process_events()
 
     def _vispy_set_current(self):
         if self._vispy_canvas is None:
@@ -611,7 +787,17 @@ class CanvasBackendDesktop(QtBaseCanvasBackend, QGLWidget):
         # Swap front and back buffer
         if self._vispy_canvas is None:
             return
-        self.swapBuffers()
+        if QT5_NEW_API:
+            ctx = self.context()
+            ctx.swapBuffers(ctx.surface())
+        else:
+            self.swapBuffers()
+
+    def _vispy_get_fb_bind_location(self):
+        if QT5_NEW_API:
+            return self.defaultFramebufferObject()
+        else:
+            return QtBaseCanvasBackend._vispy_get_fb_bind_location(self)
 
     def initializeGL(self):
         if self._vispy_canvas is None:
@@ -621,7 +807,15 @@ class CanvasBackendDesktop(QtBaseCanvasBackend, QGLWidget):
     def resizeGL(self, w, h):
         if self._vispy_canvas is None:
             return
-        self._vispy_canvas.events.resize(size=(w, h))
+        if hasattr(self, 'devicePixelRatio'):
+            # We take into account devicePixelRatio, which is non-unity on
+            # e.g HiDPI displays.
+            ratio = self.devicePixelRatio()
+            w = w * ratio
+            h = h * ratio
+        self._vispy_set_physical_size(w, h)
+        self._vispy_canvas.events.resize(size=(self.width(), self.height()),
+                                         physical_size=(w, h))
 
     def paintGL(self):
         if self._vispy_canvas is None:
