@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015, Vispy Development Team.
+# Copyright (c) Vispy Development Team. All Rights Reserved.
 # Distributed under the (new) BSD License. See LICENSE.txt for more info.
 """
 Provides classes representing different transform types suitable for
@@ -9,15 +9,15 @@ use with visuals and scenes.
 
 from .base_transform import BaseTransform  # noqa
 from .linear import (NullTransform, STTransform,  # noqa
-                     AffineTransform,  PerspectiveTransform)  # noqa
+                     MatrixTransform,  MatrixTransform)  # noqa
 from .nonlinear import LogTransform, PolarTransform  # noqa
 from .interactive import PanZoomTransform
 from .chain import ChainTransform  # noqa
 from ._util import arg_to_array, arg_to_vec4, as_vec4, TransformCache  # noqa
 from .transform_system import TransformSystem
 
-__all__ = ['NullTransform', 'STTransform', 'AffineTransform',
-           'PerspectiveTransform', 'LogTransform', 'PolarTransform',
+__all__ = ['NullTransform', 'STTransform', 'MatrixTransform',
+           'MatrixTransform', 'LogTransform', 'PolarTransform',
            'ChainTransform', 'TransformSystem', 'PanZoomTransform']
 
 transform_types = {}
