@@ -10,17 +10,24 @@ class XYZAxisVisual(LineVisual):
     x=red, y=green, z=blue.
     """
     def __init__(self, **kwargs):
-        verts = np.array([[0, 0, 0],
-                          [1, 0, 0],
-                          [0, 0, 0],
-                          [0, 1, 0],
-                          [0, 0, 0],
-                          [0, 0, 1]])
+        pos = np.array([[0, 0, 0],
+                        [1, 0, 0],
+                        [0, 0, 0],
+                        [0, 1, 0],
+                        [0, 0, 0],
+                        [0, 0, 1]])
         color = np.array([[1, 0, 0, 1],
                           [1, 0, 0, 1],
                           [0, 1, 0, 1],
                           [0, 1, 0, 1],
                           [0, 0, 1, 1],
                           [0, 0, 1, 1]])
-        LineVisual.__init__(self, pos=verts, color=color, connect='segments',
-                            method='gl', **kwargs)
+        connect = 'segments'
+        method = 'gl'
+
+        kwargs.setdefault('pos', pos)
+        kwargs.setdefault('color', color)
+        kwargs.setdefault('connect', connect)
+        kwargs.setdefault('method', method)
+
+        LineVisual.__init__(self, **kwargs)
